import time
import requests
import hashlib
import base64

import requests.packages.urllib3

requests.packages.urllib3.disable_warnings()


class AuthorisationError(ConnectionError):
    pass


def get_api_key():
    try:
        from api_key import publicKey, privateKey
        return publicKey, privateKey
    except ImportError:
        txt1 = "{} Not a valid key.\n Ask an api key at api@hexoskin.com.\n " \
               "And save in the file api_key_template.py: \n"
        raise AuthorisationError(txt1)


def get_url(user='athlete@hexoskin.com', password='not_valid_pass', apiurl='https://api.hexoskin.com/api/'):
    auth = (user, password)
    headers1 = build_header(apiurl)
    resp = requests.get(apiurl, auth=auth, headers=headers1)
    if resp.status_code != 200:
        raise AssertionError(resp.content)
    return resp.content


def download_range(user='athlete@hexoskin.com', password='not_valid_pass', filters='', domain='https://api.hexoskin.com'):
    resource = '/api/range/'
    if filters:
        resource += '?' + filters
    apiurl = domain + resource
    auth = (user, password)
    headers1 = build_header(apiurl, )
    resp = requests.get(apiurl, auth=auth, headers=headers1)
    if resp.status_code != 200:
        raise AssertionError(resp.content)
    return resp.content


def download_record(user='athlete@hexoskin.com', password='not_valid_pass', filters='', domain='https://api.hexoskin.com'):
    resource = '/api/record/'
    if filters:
        resource += '?' + filters
    apiurl = domain + resource
    auth = (user, password)
    headers1 = build_header(apiurl)
    resp = requests.get(apiurl, auth=auth, headers=headers1)
    if resp.status_code != 200:
        raise AssertionError(resp.content)
    return resp.content


def download_data(user='athlete@hexoskin.com', password='not_valid_pass', filters='',
                  accept='application/octet-stream', domain='https://api.hexoskin.com'):
    resource = '/api/data/'
    if filters:
        resource += '?' + filters
    apiurl = domain + resource
    auth = (user, password)
    headers1 = build_header(apiurl, headers={'accept': accept})
    resp = requests.get(apiurl, auth=auth, headers=headers1)
    if resp.status_code != 200:
        raise AssertionError(resp.content)
    return resp


class EmptyRequest:
    def __init__(self, get_filter=None):
        self.cached_timeranges_list = {}
        self.path = ''
        self.GET = get_filter or {}
        self.META = {}
        self.method = 'GET'


def build_header(apiurl, userpass=None, headers=None):
    publicKey, privateKey = get_api_key()
    ts = int(time.time())
    digest = hashlib.sha1('{}{}{}'.format(privateKey, ts, apiurl).encode('utf-8')).hexdigest()
    headers = headers or {}
    headers['X-HEXOTIMESTAMP'] = str(ts)
    headers['X-HEXOAPIKEY'] = publicKey
    headers['X-HEXOAPISIGNATURE'] = digest
    headers['REQUEST_URI'] = apiurl

    if userpass:
        headers['HTTP_AUTHORIZATION'] = 'Basic %s' % base64.encodebytes(userpass)
    return headers
